##LibName#Tests
##LibDescription#Funktionen zum Durchfhren von statistischen Tests:
##LibDescription# - t-Test (Einstichprobenfall)
##LibDescription# - Zwei-Stichproben-t-Test (2 unabhngige Stichproben)
##LibDescription# - Wilcoxon-Rangsummentest (2 unabhngige Stichproben)
##LibDescription# - Levene-Test (2 unabhngige Stichproben)
##LibDescription# - Chiquadrat-Anpassungstest (nur fr Normalverteilung implementiert)
##LibDescription# - Kolmogorov-Smirnov-Test (nur fr Normalverteilung implementiert)
##LibDescription# - ANOVA (auf drei Stichproben begrenzt)
##LibCopyright# Amit Ghosh

ttest <- function( X=NA, my=0, konf=0.95, plot=FALSE )
##FunctionName#ttest
##FunctionDescription#Durchfhren des t-Tests im univariaten Fall
##FunctionSyntax#ttest( X, my=0, konf=.95, plot=FALSE)
##FunctionDescription#
{
  if (!is.vector(X) | is.na(X) | length(X)<=1)
  {
     return("X muss ein Vektor mit mindestens 2 Elementen sein!")
     exit
  }
  if (!(is.vector(my) & is.numeric(my) & length(my)==1))
  {
    return("my muss ein Skalar sein!")
    exit
  }
  if (is.na(konf) | !is.numeric(konf) | !(length(konf)==1) | (konf<=0) | (konf>=1) )
  {
    return("konf muss in (0, 1) liegen!")
    exit
  }
  n <- length( X )
  df= n-1
  tq <- qt((1+konf)/2, df=df)
  diff <- mean( X ) - my
  s <- sd( X )
  t <- diff / ( s / sqrt(n) )
  sig2 <- 2*pt(abs(t), df, lower.tail=FALSE)
  konfl <- diff - tq * s / sqrt(n)
  konfu <- diff + tq * s / sqrt(n)
  namen <- c("t", "df", "Sig. (2-seitig)", "Mittlere Diff", paste("Ugr. ",100*konf,"% KI d. Diff.", sep=""), paste("Ogr. ",100*konf,"% KI d. Diff.", sep=""), row.names=NULL)
  ausgabe <- as.data.frame(cbind(round(t, 3), df, round(sig2, 3), diff, konfl, konfu))
  names(ausgabe) <- namen

  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.5,.1,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    kritW <- qt(p=(2*konf), df)
    xrange1 <- abs(t)*1.4
    xrange2 <- 2*s / sqrt(n)
    xrange <- max(xrange1, xrange2) * c(-1,1)
    yrange <- c(dt(0, df)*c(0, 1.2))
    plot(0, -5, xlim=xrange, ylim=yrange, main="t-Test", xlab="t", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[1]*1.5,-abs(t),length=50) 
    dvals <- dt(xvals, df) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    xvals <- seq(xrange[2]*1.5,abs(t),length=50) 
    dvals <- dt(xvals, df) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(dt(x, df), col="#006699", lwd=3,add=TRUE)         # Dichte t
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    text(0,yrange[2]*0.9, expression(H[0]), pos=4, col="#006699")
    segments(t, 0, t, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(t,yrange[2]*0.75, "t", pos=4, col="#FF9933")
    segments(-t, 0, -t, yrange[2], lwd=2, lty=3, col="#FF9933")     # - Wert der Teststatistik
    par(lty=1)
    text(-t,yrange[2]*0.75, "-t", pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.42, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, paste("t=",round(t,2), sep=""), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(sig2,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")
    par(opar)
  }
  return(ausgabe)
}

ttest2 <- function( X=NA, Y=NA, gruppe=NA, plot=FALSE )
##FunctionName#ttest2
##FunctionDescription#Durchfhren des Zwei-Stichproben-t-Tests bei unabhngigen Stichproben
##FunctionSyntax#ttest2( X, Y, gruppe, plot=FALSE)
##FunctionDescription#
{
  if (!is.vector(X) | is.na(X) | length(X)<=4)
  {
     return("X sollte ein Vektor mit mindestens 4 Elementen sein!")
     exit
  }
  if (is.na(gruppe))
  {
  # Y muss gegeben sein!
    if (!is.vector(Y) | is.na(Y) | length(Y)<=4)
    {
       return("Y sollte ein Vektor mit mindestens 4 Elementen sein!")
       exit
    }
  } else
  {
  # statt Y ist gruppe gegeben...
    if (!is.vector(gruppe) | is.na(gruppe) | length(gruppe)!=length(X))
    {
       return("gruppe muss ein Vektor mit gleicher Lnge wie X sein!")
       exit
    }
    countgr1 <- 0; countgr2 <- 0
    for (i in 1:length(gruppe))
    {
       if (gruppe[i]==0) countgr1 <- countgr1 + 1
       if (gruppe[i]==1) countgr2 <- countgr2 + 1
    }
    if ((countgr1+countgr2)<length(gruppe))
    {
       return("gruppe darf nur 0 und 1 enthalten!")
    }
    if (countgr1<2 | countgr2<2)
    {
       return("Jede Gruppe muss mindestens zwei Elemente enthalten!")
    }
    # Trenne Datenstze
    Y <- X[gruppe==1]
    X <- X[gruppe==0]
  }
  m <- length( X )
  n <- length( Y )
  # Levene Test
  Xs <- abs(X - mean(X)); Ys <- abs(Y - mean(Y))
  Sxs <- sd(Xs); Sys <- sd(Ys)
  Fz <- (mean(Xs)-mean(Ys))^2
  Fn <- (1/m + 1/n) * ((m - 1)*Sxs^2 + (n - 1)*Sys^2 ) / (m + n - 2)
  f <- Fz / Fn
  pLevene <- pf(f, 1, m+n-2, lower.tail=FALSE)
  # 2-Stichproben-t-Test (unabhngig)
  Sx <- sd(X); Sy <- sd(Y)
  Tz <- mean(X)-mean(Y)
  Tn <- sqrt((1/m + 1/n) * ((m - 1)*Sx^2 + (n - 1)*Sy^2 ) / (m + n - 2))
  t <- Tz / Tn
  dft <- m + n - 2
  sigT <- 2 * pt(abs(t), dft, lower.tail=FALSE)
  # Welch-Test
  Twn <- sqrt((Sx^2 / m) + (Sy^2 / n))
  tw <- Tz / Twn
  dfw <- ((Sx^2 / m) + (Sy^2 / n))^2 / ((Sx^4 / (m^2 * (m-1))) + (Sy^4 / (n^2 * (n-1))))
  sigTw <- 2 * pt(abs(tw), dfw, lower.tail=FALSE)
  # Ausgabe vorbereiten
  namen <- c("Levene-F", "Levene-Sig.", "t", "df", "Sig. (2-seitig)", "Mittlere Diff", "Std.fehler d. Diff.")
  ausgabe <- as.data.frame(cbind(c(round(f,3), NA), c(round(pLevene,3), NA), round(c(t, tw),3), c(dft, round(dfw,3)), round(c(sigT, sigTw),3), round(c(Tz, Tz),4), round(c(Tn, Twn),5)), row.names=c("t-Test", "Welch-Test"))
  names(ausgabe) <- namen
  # ggf. Graphik ausgeben
  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,2), mai=c(.5,.1,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    xrange <- abs(t)* c(-1.4,1.4)
#    xrange2 <- 2*Tn
#    xrange <- max(xrange1, xrange2) * c(-1,1)
    yrange <- c(dt(0, dft)*c(0, 1.2))

    # Ausgabe fr t-Test
    plot(0, -5, xlim=xrange, ylim=yrange, main="2-Stichproben-t-Test", xlab="t", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[1]*1.5,-abs(t),length=50) 
    dvals <- dt(xvals, dft) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    xvals <- seq(xrange[2]*1.5,abs(t),length=50) 
    dvals <- dt(xvals, dft) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(dt(x, dft), col="#006699", lwd=3,add=TRUE)         # Dichte t
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    text(0,yrange[2]*0.9, expression(H[0]), pos=4, col="#006699")
    segments(t, 0, t, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(t,yrange[2]*0.75, "t", pos=4, col="#FF9933")
    segments(-t, 0, -t, yrange[2], lwd=2, lty=3, col="#FF9933")     # - Wert der Teststatistik
    par(lty=1)
    text(-t,yrange[2]*0.75, "-t", pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.42, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, paste("t=",round(t,2), sep=""), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(sigT,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")

    # Ausgabe fr Welch-Test
    plot(0, -5, xlim=xrange, ylim=yrange, main="Welch-Test", xlab="t", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[1]*1.5,-abs(tw),length=50) 
    dvals <- dt(xvals, dfw) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    xvals <- seq(xrange[2]*1.5,abs(tw),length=50) 
    dvals <- dt(xvals, dfw) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(dt(x, dfw), col="#006699", lwd=3,add=TRUE)         # Dichte t
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    text(0,yrange[2]*0.9, expression(H[0]), pos=4, col="#006699")
    segments(tw, 0, tw, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(tw,yrange[2]*0.75, expression(t[w]), pos=4, col="#FF9933")
    segments(-tw, 0, -tw, yrange[2], lwd=2, lty=3, col="#FF9933")     # - Wert der Teststatistik
    par(lty=1)
    text(-tw,yrange[2]*0.75, expression(-t[w]), pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.42, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, substitute(t[w]==a,list(a=round(tw,2))), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(sigTw,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")

    par(opar)
  }
  return(ausgabe)
}

wilcoxon <- function( X=NA, Y=NA, gruppe=NA, plot=FALSE )
##FunctionName#wilcoxon
##FunctionDescription#Durchfhren des Wilcoxon-Rangsummentests bei 2 unabhngigen Stichproben
##FunctionSyntax#wilcoxon( X, Y, gruppe, plot=FALSE)
##FunctionDescription#
{
  if (!is.vector(X) | is.na(X) | length(X)<=4)
  {
     return("X sollte ein Vektor mit mindestens 4 Elementen sein!")
     exit
  }
  if (is.na(gruppe))
  {
  # Y muss gegeben sein!
    if (!is.vector(Y) | is.na(Y) | length(Y)<=4)
    {
       return("Y sollte ein Vektor mit mindestens 4 Elementen sein!")
       exit
    }
  } else
  {
  # statt Y ist gruppe gegeben...
    if (!is.vector(gruppe) | is.na(gruppe) | length(gruppe)!=length(X))
    {
       return("gruppe muss ein Vektor mit gleicher Lnge wie X sein!")
       exit
    }
    countgr1 <- 0; countgr2 <- 0
    for (i in 1:length(gruppe))
    {
       if (gruppe[i]==0) countgr1 <- countgr1 + 1
       if (gruppe[i]==1) countgr2 <- countgr2 + 1
    }
    if ((countgr1+countgr2)<length(gruppe))
    {
       return("gruppe darf nur 0 und 1 enthalten!")
    }
    if (countgr1<2 | countgr2<2)
    {
       return("Jede Gruppe muss mindestens zwei Elemente enthalten!")
    }
    # Trenne Datenstze
    Y <- X[gruppe==1]
    X <- X[gruppe==0]
  }
  m <- length( X )
  n <- length( Y )
  N <- m + n
  pool <- c(X, Y)
  raengepool <- rank(pool)
  raengeX <- raengepool[1:m]
  raengeY <- raengepool[(m+1):N]
  Wn <- sum(raengeX)
  EWn <- (m * (N + 1)) / 2
  VarWn <- (m * n * (N + 1)) / 12
  z <- (Wn - EWn) / sqrt(VarWn)
  pWilcoxon <- 2 * pnorm(abs(z), lower.tail=FALSE)
  # Ausgabe vorbereiten
  namen <- c("Wilcoxon w", "z", "Asym. Sig. (2-seitig)")
  ausgabe <- as.data.frame(cbind(round(Wn,3), round(z,3), round(pWilcoxon, 3)))
  names(ausgabe) <- namen
  # ggf. Graphik ausgeben
  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.5,.1,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    xrange <- abs(z)* c(-1.4,1.4)
    yrange <- c(dnorm(0)*c(0, 1.2))

    # Ausgabe fr Wilcoxon
    plot(0, 0, type="n", xlim=xrange, ylim=yrange, main="Wilcoxon-Rangsummentest", xlab="z", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[1]*1.5,-abs(z),length=50) 
    dvals <- dnorm(xvals) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    xvals <- seq(xrange[2]*1.5,abs(z),length=50) 
    dvals <- dnorm(xvals) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(dnorm(x), col="#006699", lwd=3,add=TRUE)         # Dichte t
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    text(0,yrange[2]*0.9, expression(H[0]), pos=4, col="#006699")
    segments(z, 0, z, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(z,yrange[2]*0.75, "z", pos=4, col="#FF9933")
    segments(-z, 0, -z, yrange[2], lwd=2, lty=3, col="#FF9933")     # - Wert der Teststatistik
    par(lty=1)
    text(-z,yrange[2]*0.75, "-z", pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.42, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, paste("z=",round(z,2), sep=""), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(pWilcoxon,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")
    par(opar)
  }
  return(ausgabe)
}

levene <- function( X=NA, Y=NA, gruppe=NA, plot=FALSE )
##FunctionName#levene
##FunctionDescription#Durchfhren des Levene-Tests bei 2 unabhngigen Stichproben
##FunctionSyntax#levene( X, Y, gruppe, plot=FALSE)
##FunctionDescription#
{
  if (!is.vector(X) | is.na(X) | length(X)<=4)
  {
     return("X sollte ein Vektor mit mindestens 4 Elementen sein!")
     exit
  }
  if (is.na(gruppe))
  {
  # Y muss gegeben sein!
    if (!is.vector(Y) | is.na(Y) | length(Y)<=4)
    {
       return("Y sollte ein Vektor mit mindestens 4 Elementen sein!")
       exit
    }
  } else
  {
  # statt Y ist gruppe gegeben...
    if (!is.vector(gruppe) | is.na(gruppe) | length(gruppe)!=length(X))
    {
       return("gruppe muss ein Vektor mit gleicher Lnge wie X sein!")
       exit
    }
    countgr1 <- 0; countgr2 <- 0
    for (i in 1:length(gruppe))
    {
       if (gruppe[i]==0) countgr1 <- countgr1 + 1
       if (gruppe[i]==1) countgr2 <- countgr2 + 1
    }
    if ((countgr1+countgr2)<length(gruppe))
    {
       return("gruppe darf nur 0 und 1 enthalten!")
    }
    if (countgr1<2 | countgr2<2)
    {
       return("Jede Gruppe muss mindestens zwei Elemente enthalten!")
    }
    # Trenne Datenstze
    Y <- X[gruppe==1]
    X <- X[gruppe==0]
  }
  m <- length( X )
  n <- length( Y )
  # Levene Test
  Xs <- abs(X - mean(X)); Ys <- abs(Y - mean(Y))
  Sxs <- sd(Xs); Sys <- sd(Ys)
  Fz <- (mean(Xs)-mean(Ys))^2
  Fn <- (1/m + 1/n) * ((m - 1)*Sxs^2 + (n - 1)*Sys^2 ) / (m + n - 2)
  f <- Fz / Fn
  pLevene <- pf(f, 1, m+n-2, lower.tail=FALSE)
  # Ausgabe vorbereiten
  namen <- c("Levene-F", "Levene-Sig.")
  ausgabe <- as.data.frame(cbind(round(f,3), round(pLevene,3)))
  names(ausgabe) <- namen
  # ggf. Graphik ausgeben
  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.5,.2,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    xrange <- c(0,max(2, f*1.4))
    yrange <- c(0, 2)

    # Ausgabe fr Levene-Test
    plot(0, -5, xlim=xrange, ylim=yrange, main="Levene-Test", xlab="f", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[2]*1.5, f, length=50) 
    dvals <- df(xvals, 1, m + n - 2) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(df(x, 1, m + n - 2), .0001, xrange[2], 1000, col="#006699", lwd=3,add=TRUE)         # Dichte F
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    mtext(expression(H[0]), side=2, col="#006699")
    segments(f, 0, f, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(f,yrange[2]-(yrange[2]/xrange[2])*f, "f", pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.62, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, paste("f=",round(f,2), sep=""), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(pLevene,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")
    par(opar)
  }
  return(ausgabe)
}

chisqanpnorm <- function ( X=NA, klass=5, my=NA, sigma=NA, plot=FALSE )
##FunctionName#chianpnorm
##FunctionDescription#Durchfhren des t-Tests im univariaten Fall
##FunctionSyntax#chisqanpnorm( X, klass=5, my=NA, sigma=NA, plot=FALSE)
##FunctionDescription#
{
  m <- 0
  if (!is.vector(X) | is.na(X) | length(X)<=1)
  {
     return("X muss ein Vektor mit mindestens 2 Elementen sein!")
     exit
  }
  if (is.na(my))
  {
     my <- mean( X )
     m <- m + 1
  }
  if (!(is.vector(my) & is.numeric(my) & length(my)==1))
  {
    return("my muss ein Skalar sein!")
    exit
  }
  if (!(is.vector(klass) & is.numeric(klass) & length(klass)==1) & (klass>2))
  {
    return("klass muss ein Skalar sein!")
    exit
  }
  if (is.na(sigma))
  {
     sigma <- sd( X )
     m <- m + 1
  }
  if (!(is.vector(sigma) & is.numeric(sigma) & length(sigma)==1))
  {
    return("sigma muss ein Skalar sein!")
    exit
  }
  n <- length( X )
  df= klass - 1 - m
  if (df<1)
  {
     return("Zu wenig Klassen!")
     exit
  }
  klassgrpr <- seq(0, 1, length=(klass+1))
  klassgr <- qnorm(klassgrpr, my, sigma)
  pi <- pnorm(q=klassgr[2:(klass+1)], my, sigma)-pnorm(klassgr[1:klass], my, sigma)
  nischl <- pi * n
  chin <- rep(NA, klass)
  for (i in 2:(klass+1))
  {
    ni <- length(X[(klassgr[i-1]<X & X<=klassgr[i])])
    chin[i-1] <- ((ni - nischl[i-1])^2) / nischl[i-1]
  }  
  chisq <- sum(chin)
  pVal <- pchisq(chisq, df, lower.tail=FALSE)
  namen <- c("Chi-Quadrat", "df", "Asymp. Sig.")
  ausgabe <- as.data.frame(cbind(round(chisq, 3), df, round(pVal, 3)))
  names(ausgabe) <- namen

  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.5,.2,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    xrange <- c(0, chisq*2)
    yrange <- c(dt(0, df)*c(0, 1.2))
    plot(0, 0, type="n", xlim=xrange, ylim=yrange, main="Chisq-Anpassungs-Test", xlab="chisq", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(chisq, xrange[2]*1.5, length=50) 
    dvals <- dchisq(xvals, df) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(dchisq(x, df), 0, xrange[2], 1000, col="#006699", lwd=3,add=TRUE)         # Dichte chisq
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    mtext(expression(H[0]), side=2, col="#006699")
    segments(chisq, 0, chisq, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(chisq,yrange[2]*0.75, expression(chi^2), pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]*.62, yrange[2]*.83,xrange[2]*1.02, yrange[2], border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]*.9, yrange[2]*.95, paste("chisq=",round(chisq,2), sep=""), pos=2)
    text(xrange[2]*.9, yrange[2]*.88, paste("p-Wert=",round(pVal,2), sep=""), pos=2)
    segments(xrange[2]*.95, yrange[2]*.91, xrange[2]*.95, yrange[2]*.99, lwd=2, col="#FF9933")
    rect(xrange[2]*.9, yrange[2]*.84, xrange[2], yrange[2]*.92, density=20, col="#FF9933", border="#AAAAAA")
    par(opar)
  }
  return(ausgabe)
}

ksnorm <- function (X, plot=FALSE)
{
##FunctionName#ksnorm
##FunctionDescription#Durchfhren des Kolmogorov-Smirnov-Tests zum Testen auf Normalverteilung
##FunctionSyntax#ksnorm( X, plot=FALSE)
##FunctionDescription#

  pkstwo <- function(z, tol = 1e-06) {
    if (is.numeric(z))
      x <- as.vector(z)
    else stop("Interner Fehler!")
    p <- rep(0, length(z))
    p[is.na(z)] <- NA
    IND <- which(!is.na(z) & (z > 0))
    if (length(IND) > 0)
      p[IND] <- .C("pkstwo", as.integer(length(z)), p = as.double(z[IND]), as.double(tol), PACKAGE = "ctest")$p
    return(p)
  }

  if (!is.vector(X) | is.na(X) | length(X)<4)
  {
     return("X muss ein Vektor mit mindestens 4 Elementen sein!")
     exit
  }
  n <- length(X)
  xsort <- sort(X)
  countxdiff <- 0
  pos <- 1
  repeat
  {
    currentxval <- xsort[pos]
    repeat
    {
      pos <- pos + 1
      if ((pos > n) | (xsort[pos] > currentxval)) break
    }
    countxdiff <- countxdiff + 1
    if (pos > n) break
  }
  xdiff <- rep(NA, countxdiff)
  xdiffcount <- rep(NA, countxdiff)
  pos <- 1
  posdiff <- 1
  repeat
  {
    xdiff[posdiff] <- xsort[pos]
    xdiffcount[posdiff] <- 1
    repeat
    {
      pos <- pos + 1
      if ((pos > n) | (xsort[pos] > xdiff[posdiff]))
        break
      else
        xdiffcount[posdiff] <- xdiffcount[posdiff] + 1
    }
    posdiff <- posdiff + 1
    if (pos > n) break
  }
  meanx <- mean(X)
  sdx <- sd(X)
  dnplus <- rep(NA, countxdiff)
  dnminus <- rep(NA, countxdiff)
  fval <- rep(NA, countxdiff)
  problow <- rep(NA, countxdiff)
  probhigh <- rep(NA, countxdiff)
  for (i in 1:countxdiff)
  {
     if (i>1)
       problow[i] <- (1 / n) * sum(xdiffcount[1:(i-1)])
     else
       problow[i] <- 0
     probhigh[i] <- (1 / n) * sum(xdiffcount[1:i])
     fval[i] <- pnorm(xdiff[i], meanx, sdx)
     dnplus[i] <- fval[i] - probhigh[i]
     dnminus[i] <- fval[i] - problow[i]
  }
  k <- max(abs(dnminus), abs(dnplus))
  namen <- c("xi", "F*(xi)", "Fn(xi+)", "Fn(xi-)", "Dn+", "Dn-")
  ret1 <- as.data.frame(cbind(xdiff, round(fval, 3), round(probhigh,3), round(problow,3), round(abs(dnplus),3), round(abs(dnminus),3)))
  names(ret1) <- namen
  z <- sqrt(n) * k
  pval <- 1 - pkstwo(z)
  ret <- list(Tabelle=ret1, k=round(k,3), z=round(z,3), p=round(pval,3))
    if (plot)
  {
    xrange <- c(min(X)-(max(X)-min(X))*.5, max(X)+(max(X)-min(X))*.5)
    xspan <- xrange[2]-xrange[1]
    yrange <- c(0, 1)
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.7,.7,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    plot(c(xrange[1], xdiff, xrange[2]), c(0, (1 / n) * cumsum(xdiffcount), 1), type="s",main="K-S-Test", xlab="x", ylab="F(x)")
    curve(pnorm(x, meanx, sdx), col="#006699", lwd=3, add=TRUE)
    # Legende
    rect(xrange[2]-xspan*.27, yrange[2]*.07, xrange[2], yrange[2]*.22, border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]-xspan*.02, yrange[2]*.17, paste("z=",round(z,2), sep=""), pos=2)
    text(xrange[2]-xspan*.02, yrange[2]*.11, paste("p-Wert=",round(pval,2), sep=""), pos=2)
    par(opar)
  }
  return(ret)
}

sanova <- function (X=NA, gruppe=NA, plot=FALSE) 
{
##FunctionName#sanova
##FunctionDescription#Durchfhren der (einfachen) ANOVA
##FunctionSyntax#sanova( X, gruppe, plot=FALSE)
##FunctionDescription#

  if (!is.vector(X) | is.na(X) | length(X)<=1)
  {
     return("X muss ein Vektor mit mindestens 6 Elementen sein!")
     exit
  }
  if (!is.vector(gruppe) | is.na(gruppe) | length(gruppe)<=1)
  {
     return("gruppe muss ein Vektor mit mindestens 6 Elementen sein!")
     exit
  }
  if (is.na(X) | is.na(gruppe))
    stop("Paramter wurden unvollstndig spezifiziert!")
  if (!(length(X) == length(gruppe)))
    stop("X und gruppe mssen die gleiche Lnge haben!")
  c <- 3
  n0 <- length(gruppe[gruppe==0])
  n1 <- length(gruppe[gruppe==1])
  n2 <- length(gruppe[gruppe==2])
  n3 <- length(gruppe[gruppe==3])
  nmax <- max(n0, n1, n2, n3)
  if (!(n0 + n1 + n2 + n3 == length(X)))
    stop("gruppe ist ungltig!")
  if (n0 + n1 + n2 == length(X))
  {
    if (n0<2 | n1<2 | n2<2)
      stop("In mindestens einer Stichprobe ist der Stichprobenumfang zu gering!")
    x <- cbind(c(X[gruppe==0], rep(NA, nmax-n0)), c(X[gruppe==1], rep(NA, nmax-n1)), c(X[gruppe==2], rep(NA, nmax-n2)))
    ni <- c(n0, n1, n2)
  }
  else
  {
     if (n1 + n2 + n3 == length(X))
     {
       if (n1<2 | n2<2 | n3<2)
         stop("In mindestens einer Stichprobe ist der Stichprobenumfang zu gering!")
       x <- cbind(c(X[gruppe==1], rep(NA, nmax-n1)), c(X[gruppe==2], rep(NA, nmax-n2)), c(X[gruppe==3], rep(NA, nmax-n3)))
           ni <- c(n1, n2, n3)
     }
     else
       stop("gruppe ist ungltig!")
  }
  if (length(ni[ni<2])>0)
    stop("In mindestens einer Stichprobe ist der Stichprobenumfang zu gering!")
  c <- dim(x)[2]
  n <- sum(ni)
  SSB <- 0
  SSW <- 0
  sumx <- 0
  meani <- rep(NA, c)
  for (i in 1:c)
  {
    meani[i] <- mean(x[1:ni[i], i])
    for (j in 1:ni[i])
    {
       SSW <- SSW + (x[j, i] - meani[i])^2
       sumx <- sumx + x[j, i]
    }
  }
  meanx <- sumx / sum(n)
  SSB <- sum(ni * (meani - meanx)^2)
  SST <- SSB + SSW
  MSSB <- 1 / (c - 1) * SSB
  MSSW <- 1 / (n - c) * SSW
  Fval <- MSSB / MSSW
  pval <- pf(Fval, df1=c-1, df2=n-c, lower.tail = FALSE)
  ret <- as.data.frame(cbind(c("Zwischen den Gruppen", "Innerhalb der Gruppen", "Total"), round(c(SSB, SSW, SST),3), c(c-1, n-c, n-1), round(c(MSSB, MSSW, NA),3), round(c(Fval, NA, NA),3), round(c(pval, NA, NA),3)))
  namen <- c("Bezeichnung", "Quadratsumme", "df", "Mittel d. Quad.", "F", "Sig.")
  names(ret) <- namen
  if (plot==TRUE)
  {
    opar <- par(no.readonly=TRUE)
    par(mfcol=c(1,1), mai=c(.5,.2,.5,.1), mgp=c(2.5,1,0), bg="#CCCCCC", col="#FFFFFF", col="#000000", col.main="#003366")
    xrange <- c(0,max(2, Fval*1.4))
    xspan <- xrange[2]-xrange[1]
    yrange <- c(0, 2)

    # Ausgabe fr Levene-Test
    plot(0, -5, xlim=xrange, ylim=yrange, main="ANOVA (einfach)", xlab="f", ylab="", axes=FALSE)
    # Shader
    xvals <- seq(xrange[2]*1.5, Fval, length=50) 
    dvals <- df(xvals, c-1, n-c) 
    polygon(c(xvals,rev(xvals)),c(rep(0,50),rev(dvals)),density=10, col="#FF9933", border="#CCCCCC")
    # End Shader
    axis(1)
    curve(df(x, c-1, n-c), .0001, xrange[2], 1000, col="#006699", lwd=3,add=TRUE)         # Dichte F
    segments(0, 0, 0, yrange[2], col="#006699")             # H0
    mtext(expression(H[0]), side=2, col="#006699")
    segments(Fval, 0, Fval, yrange[2], lwd=2, col="#FF9933")          # Wert der Teststatistik
    text(Fval,yrange[2]-(yrange[2]/xrange[2])*Fval, "f", pos=4, col="#FF9933")
    # Legende
    rect(xrange[2]-xspan*.27, yrange[2]*.78, xrange[2], yrange[2]*.93, border="#006699", col="#AAAAAA", lwd=2)
    text(xrange[2]-xspan*.08, yrange[2]*.89, paste("f=",round(Fval,2), sep=""), pos=2)
    text(xrange[2]-xspan*.08, yrange[2]*.83, paste("p-Wert=",round(pval,2), sep=""), pos=2)
    segments(xrange[2]-xspan*.0325, yrange[2]*.86, xrange[2]-xspan*.0325, yrange[2]*.925, lwd=2, col="#FF9933")
    rect(xrange[2]-xspan*.06, yrange[2]*.79, xrange[2]-xspan*.005, yrange[2]*.855, density=20, col="#FF9933", border="#AAAAAA")
    par(opar)
  }

  return(ret)
}
